package com.sooka.sponest.middleground.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.sooka.sponest.middleground.remoteapi.service.center.dahuaLog.RemoteDahuaLogService;
import com.sooka.sponest.middleground.remoteapi.service.center.gpsTank.RemoteGpsTankService;
import com.sooka.sponest.middleground.scheduleTask.StaticScheduleTask;
import com.sooka.sponest.middleground.scheduleTask.entity.GasTankDevices;
import com.sooka.sponest.middleground.util.LoginUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.TrustSelfSignedStrategy;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.client.LaxRedirectStrategy;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import static org.springframework.web.bind.annotation.RequestMethod.GET;


@Slf4j
@Controller
@RequestMapping("/gps/tank")
public class MonitorController {
    
    protected static final Logger logger = LoggerFactory.getLogger(StaticScheduleTask.class);
    @Value("${sooka.equip_gps.ip:}")
    public String ip;
    @Value("${sooka.equip_gps.login_name:}")
    public String loginName;
    @Value("${sooka.equip_gps.password:}")
    public String password;
    @Value("${sooka.equip_gps.username:}")
    public String username;
    @Resource
    public RemoteGpsTankService remoteGpsTankService;
    @Resource
    public RemoteDahuaLogService remoteDahuaLogService;


    /**
     * 执行每1小时对接大华云台日志定时任务
     *
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/getDHLog", method = GET)
    @ResponseBody
    private void configureBlackTasks() {
        logger.info("执行每1小时对接大华云台日志定时任务=>{}", DateUtils.dateTimeNow());
        remoteDahuaLogService.configureWhiteTasks();
    }


    /**
     * 根据用户名查询设备列表,新增煤气罐设备信息
     *
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/queryMonitorList", method = GET)
    @ResponseBody
    public AjaxResult queryMonitorList() {
        AjaxResult ajaxResult = AjaxResult.success();
        String token = null;
        try {
            token = LoginUtil.login(ip, loginName, password);
            HttpPost httpPost = new HttpPost(ip + "/webapi?action=querymonitorlist&token=" + token);
            String requestBody = "{\r\n\t\"username\": \"" + username + "\"}";
            JSONObject jsonObject = getJsonObject(requestBody, httpPost);
            if (jsonObject != null) {
                ajaxResult = remoteGpsTankService.gpsTankInsertBatch(jsonObject);
                return ajaxResult;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ajaxResult;
    }

    /**
     * 对接北斗卫星定位设备车辆轨迹和行程
     *
     * @return
     * @throws Exception
     */
    public void configureGpsTank() throws Exception {
        // 创建日期格式化对象
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 获取当前日期
        LocalDateTime nowDateTime = LocalDateTime.now();
        String startOfDay;
        String endOfDay;
        //获取token
        String token = LoginUtil.login(ip, loginName, password);
        //获取设备id列表
        List<GasTankDevices> deviceList = new ArrayList<>();
        AjaxResult resultOne = remoteGpsTankService.selectGasTankDevicesNewEndTime();
        if (resultOne.get("code").equals(200)) {
            deviceList = JSON.parseArray(JSON.toJSONString(resultOne.get("data")), GasTankDevices.class);
        } else {
            logger.error("查询北斗卫星定位设备列表失败");
        }
        //按时间查询历史轨迹
        HttpPost trajectoriesHttpPost = new HttpPost(ip + "/webapi?action=querytracks&token=" + token);
        //查询行程
        HttpPost travelHttpPost = new HttpPost(ip + "/webapi?action=querytrips&token=" + token);
        for (GasTankDevices device : deviceList) {
            LocalDateTime date = LocalDateTime.now();
            //结束时间为当前时间
            endOfDay = formatter.format(date);
            //开始时间为当前时间减12分钟后的时间
            LocalDateTime endOfDayDateTime = date.plusMinutes(-12);
            startOfDay = endOfDayDateTime.format(formatter);
            // 入参报文
            String requestBody = "{\r\n\t\"deviceid\": \"" + device.getDeviceid() + "\",\r\n\t\"begintime\": \"" + startOfDay + "\",\r\n\t\"endtime\": \"" + endOfDay + "\",\r\n\t\"timezone\": \"8\"}";
            JSONObject trajectoryObj = getJsonObject(requestBody, trajectoriesHttpPost);

            AjaxResult trajectoryResult = remoteGpsTankService.remoteTrajectoryInsertBatch(trajectoryObj);
            if (!trajectoryResult.get("code").equals(200)) {
                logger.info("北斗卫星定位车辆gps历史轨迹对接,数据新增失败");
            }
            JSONObject mileageObject = getJsonObject(requestBody, travelHttpPost);
            AjaxResult mileageResult = remoteGpsTankService.remoteMileageInsertBatch(mileageObject);
            if (!mileageResult.get("code").equals(200)) {
                logger.info("北斗卫星定位车辆gps历史行程对接,数据新增失败");
            }
        }
        logger.info("执行对接北斗卫星定位设备车辆轨迹和行程定时任务结束");
    }


    private static JSONObject getJsonObject(String requestBody, HttpPost httpPost) throws Exception {
        SSLContext sslContext = SSLContextBuilder.create()
                .loadTrustMaterial(new TrustSelfSignedStrategy())
                .build();
        HostnameVerifier hostnameVerifier = NoopHostnameVerifier.INSTANCE;
        HttpClient httpClient = HttpClients.custom()
                .setSSLContext(sslContext)
                .setSSLHostnameVerifier(hostnameVerifier)
                .setRedirectStrategy(new LaxRedirectStrategy())
                .build();
        StringEntity requestEntity = new StringEntity(requestBody, ContentType.APPLICATION_JSON);
        httpPost.setEntity(requestEntity);
        HttpResponse response = httpClient.execute(httpPost);
        HttpEntity responseEntity = response.getEntity();
        String responseBody = EntityUtils.toString(responseEntity);
        JSONObject jsonObject = JSONObject.parseObject(responseBody);
        return jsonObject;
    }
}
