package com.sooka.sponest.middleground.hwMeeting.service.impl;

import com.dahuatech.hutool.core.util.RandomUtil;
import com.dahuatech.hutool.http.HttpRequest;
import com.dahuatech.hutool.json.JSONArray;
import com.dahuatech.hutool.json.JSONObject;
import com.dahuatech.hutool.json.JSONUtil;
import com.sooka.sponest.middleground.hwMeeting.service.HwMeetingService;
import com.sooka.sponest.middleground.util.HmacSHA256;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
public class HwMeetingServiceImpl implements HwMeetingService {

    private static final Logger log = LoggerFactory.getLogger(HwMeetingServiceImpl.class);

    /**服务器地址**/
    @Value("${meeting.serverUrl:}")
    public String serverUrl;

    /**appId**/
    @Value("${meeting.appId:}")
    public String appId;

    /**第三方平台userId**/
    @Value("${meeting.userId:}")
    public String userId;

    /**执行AppID鉴权地址**/
    @Value("${meeting.appAuth:}")
    public String appAuth;

    /**appKey**/
    @Value("${meeting.appKey:}")
    public String appKey;

    /**vmr**/
    @Value("${meeting.vmr:}")
    public String vmr;

    /**查询在线会议**/
    @Value("${meeting.onlineConferences:}")
    public String onlineConferences;

    /**创建会议**/
    @Value("${meeting.conferences:}")
    public String conferences;

    /**获取页面免登陆跳转的nonce信息**/
    @Value("${meeting.getNonce:}")
    public String getNonce;



    /**
     * appId鉴权方法 获取token
     */
    @Override
    public String appauth() {
        //64位随机字符串
        String nonce = RandomUtil.randomString(64);
        log.info("nonce:{}", nonce);
        //鉴权信息
        String authorization = getAuthorization(appId, userId, nonce);
        log.info("authorization:{}", authorization);
        //body
        JSONObject json = new JSONObject();
        json.put("appId", appId);
        json.put("expireTime", 0);
        json.put("nonce", nonce);
//        json.put("userId", userId);
        json.put("clientType", 72);
        //请求结果
        String result = HttpRequest.post(serverUrl + appAuth)
                .header("Authorization", authorization)
                .header("Content-Type", "application/json")
                .body(json)
                .execute().body();
        //打印
        log.info("result:{}", result);
        return JSONUtil.parseObj(result).get("accessToken").toString();
    }

    /**
     * 构成Authorization
     */
    @Override
    public String getAuthorization(String appId, String userId, String nonce) {
        String data = appId + ":" + userId + ":" + 0 + ":" + nonce;
        String authorization = "HMAC-SHA256 signature=" + HmacSHA256.encode(data, appKey);
        return authorization;
    }

    /**
     * 绑定给当前创会帐号的VMR_ID。通过查询云会议室及个人会议ID接口获取。
     */
    @Override
    public String vmrID(String token) {
        //body
        String result = HttpRequest.get(serverUrl + vmr)
                .header("X-Access-Token", token)
                .header("Content-Type", "application/json")
                .execute().body();
        log.info("vmr：{}", result);
        JSONArray rooms = JSONUtil.parseArray(JSONUtil.parseObj(result).get("data"));
        log.info("rooms.get(0)：{}", rooms.get(0));
        log.info("JSONUtil.parseObj(rooms.get(0))：{}", JSONUtil.parseObj(rooms.get(0)));
        log.info("vmrID：{}", JSONUtil.parseObj(rooms.get(0)).get("vmrId"));
        return JSONUtil.parseObj(rooms.get(0)).get("id").toString();
    }

    /**
     * 查询在线会议列表
     */
    @Override
    public String hasConferences(){
        String token = appauth();
        String result = HttpRequest.get(serverUrl + onlineConferences)
                .header("X-Access-Token", token)
                .header("Content-Type", "application/json")
                .execute().body();
        log.info("查询在线会议列表：{}", result);
        return result;
    }

    /**
     * 创建会议
     */
    @Override
    public String conferences(String subject) {
        String token = appauth();
        String vmrID = vmrID(token);
        //body
        JSONObject json = new JSONObject();
        /**会议的媒体类型。Voice：语音会议,HDVideo：视频会议**/
        json.put("mediaTypes", "HDVideo");
        /**
         * 会议持续时长，单位分钟。默认30分钟。
         * 最大1440分钟（24小时），最小15分钟。
         * */
        json.put("length", 15);
        /**会议主题。最多128个字符。**/
        json.put("subject", subject);
        /**
         * 是否使用云会议室或者个人会议ID召开预约会议。默认0。
         * 0：不使用云会议室或者个人会议ID
         * 1：使用云会议室或者个人会议ID
         * */
        json.put("vmrFlag", 1);
        /**绑定给当前创会帐号的VMR ID。通过查询云会议室及个人会议ID接口获取。**/
        json.put("vmrID", vmrID);
        //请求
        String result = HttpRequest.post(serverUrl + conferences)
                .header("X-Access-Token", token)
                .header("Content-Type", "application/json")
                .body(json)
                .execute().body();
        log.info("创建会议：{}", result);
        return result;
    }

    /**
     * 获取页面免登陆跳转的nonce信息
     */
    public String getNonce(){
        String result = HttpRequest.post(serverUrl + getNonce)
                .header("X-Access-Token", appauth())
                .header("Content-Type", "application/json")
                .execute().body();
        log.info("获取页面免登陆跳转的nonce信息：{}", result);
        Object nonce = JSONUtil.parseObj(result).get("nonce");
        return nonce !=null ? nonce.toString() : "";
    }

}
