package com.sooka.sponest.middleground.monitoringEquipment.sennor.service.impl;

import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.constant.HttpStatus;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.sooka.sponest.middleground.monitoringEquipment.sennor.api.API;
import com.sooka.sponest.middleground.monitoringEquipment.sennor.service.SennorService;
import com.sooka.sponest.middleground.remoteapi.service.center.iot.RemoteIotService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
public class SennorServiceImpl implements SennorService {

    private static final Logger log = LoggerFactory.getLogger(SennorServiceImpl.class);

    /**精讯物联网秘钥**/
    @Value("${monitoringEquipment.sennor.sennorSecretKey:}")
    public String sennorSecretKey;

    /**精讯物联网服务地址**/
    @Value("${monitoringEquipment.sennor.sennorUrl:}")
    public String sennorUrl;

    @Resource
    public RemoteIotService remoteIotService;



    //这个方法应该用定时任务三分钟调用一次
    @Override
    public AjaxResult getDeviceInfo(String deviceNumber) {
        //先请求
        StringBuffer URI = new StringBuffer(sennorUrl + API.getDeviceInfo);
        URI.append("?secretKey=").append(sennorSecretKey);
        URI.append("&deviceNumber=").append(deviceNumber);
        String result = HttpRequest.get(URI.toString())
                .header("Content-Type", "application/json")
                .execute().body();
        log.info("Sennor 收到数据 ==> {}", result);
        //后推送
        JSONObject object;
        String res;
        try {
            object = JSONObject.parseObject(result);
            AjaxResult ajaxResult = remoteIotService.saveSennorDeviceData(object);
            log.info("数据推送成功! data ===> {} , result ===> {}",result, ajaxResult);
            return AjaxResult.success("数据推送成功!",ajaxResult);
        }catch(Exception e){
            e.printStackTrace();
            log.error("数据推送失败! data ===> {} , result ===> {}",result, e.getMessage());
            return AjaxResult.error("数据推送失败!",result);
        }

    }

    @Override
    public JSONArray getDevideCodeMapByTypes(String types) {
        //先请求
        AjaxResult ajaxResult = remoteIotService.getDevideCodeMapByTypes(types);
        try {
            if(ajaxResult.get("code").equals(HttpStatus.SUCCESS)){
                return JSONUtil.parseArray(ajaxResult.get("data"));
            }
        }catch(Exception e){
            e.printStackTrace();
            log.error("服务端数据校验失败! data ===> {} , result ===> {}", JSONUtil.toJsonStr(ajaxResult.get("data")), e.getMessage());
        }
        return new JSONArray();
    }
}
