package com.sooka.sponest.middleground.remoteapi.service.center.iot;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.api.domain.SysFile;
import com.sooka.sponest.middleground.remoteapi.fallback.center.iot.RemoteIotBaseServiceFallbackFactory;
import com.sooka.sponest.middleground.remoteapi.service.ModulesServiceNameContants;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.multipart.MultipartFile;

@FeignClient(
        contextId = "remoteIotService"
        , value = ModulesServiceNameContants.CENTER_MONITOR_IOT
        , fallbackFactory = RemoteIotBaseServiceFallbackFactory.class
//        ,url = "${sooka.service.monitor-iot}"
)
public interface RemoteIotService {


    /**
     * 【水流速推送】向监控中心推送水流速采集数据
     *
     * @param json
     * @return AjaxResult
     * @author limeng
     * @date 2023/7/31 16:27
     **/
    @PostMapping("/device/saveWaterMonitoringData")
    public AjaxResult saveWaterMonitoringData(@RequestBody JSONObject json);


    /**
     * 【水压】接收城安盛邦 水压传感器数据方法
     */
    @PostMapping("/device/saveWaterPressureDeviceData")
    public AjaxResult saveWaterPressureDeviceData(@RequestBody JSONObject json);

    /**
     * 【物联网】接收精讯传感器：大气、水质、土壤墒情数据通用方法
     */
    @ApiOperation(value = "接收精讯传感器数据通用方法", notes = "接收精讯传感器数据通用方法")
    @PostMapping("/device/saveSennorDeviceData")
    public AjaxResult saveSennorDeviceData(@RequestBody JSONObject json);

    /**
     * 【水流速】根据设备类型查询监控设备Map
     */
    @ApiOperation(value = "接收传感器水质数据", notes = "接收传感器水质数据")
    @PostMapping("/device/getDevideCodeMapByTypes")
    public AjaxResult getDevideCodeMapByTypes(@RequestBody String types);

    /**
     * 【病虫害】接收病虫害传感器数据方法
     */
    @ApiOperation(value = "接收病虫害传感器数据", notes = "接收病虫害传感器数据")
    @PostMapping("/device/saveInsectPestsDeviceData")
    public AjaxResult saveInsectPestsDeviceData(@RequestBody JSONObject json);

    /**
     * 病虫害文件上传
     */
    @PostMapping(value = "/centerMonitorFeign/upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public R<SysFile> upload(MultipartFile file);

    /**
     * 传感器设备定时任务
     */
    @ApiOperation(value = "传感器设备定时任务", notes = "传感器设备定时任务")
    @GetMapping("/device/quartz/updateDeviceStatusQuartz")
    public AjaxResult updateDeviceStatusQuartz();

}
